do
  local _parent_0 = Physical
  local _base_0 = {
    walk = function(self, dt, v)
      if v == 0 or (self.vx ~= 0 and lume.sign(self.vx) ~= lume.sign(v)) then
        self.currentDrag = self.horizontalDrag
      else
        self.currentDrag = 0
      end
      self.vx = self.vx + (v * self.walkAcceleration * dt)
    end,
    jump = function(self)
      if self.onGround then
        self.vy = -self.baseJumpPower - self.additionalJumpPower * (math.abs(self.vx) / self.horizontalMaxSpeed)
        self.jumping = true
        self.animation.jump:gotoFrame(2)
        return sound.playerJump:play()
      elseif self.canDoubleJump then
        if self.vy > 0 then
          self.vy = 0
        end
        self.vy = self.vy - (self.doubleJumpPower + self.additionalJumpPower * (math.abs(self.vx) / self.horizontalMaxSpeed))
        self.canDoubleJump = false
        self.jumping = true
        self.animation.jump:gotoFrame(2)
        return sound.playerDoubleJump:play()
      end
    end,
    endJump = function(self)
      self.jumping = false
    end,
    dive = function(self)
      self.diving = true
      self.vy = self.diveMaxSpeed * .3
    end,
    endDive = function(self)
      self.diving = false
    end,
    update = function(self, dt)
      _parent_0.update(self, dt)
      if self.hasBubble then
        self.timeScale = self.bubbleSpeedMultiplier
      else
        self.timeScale = 1
      end
      self.onGroundPrevious = self.onGround
      self.onGround = false
      if self.vx < 0 then
        self.vx = self.vx + (self.currentDrag * dt * self.timeScale)
        if self.vx > 0 then
          self.vx = 0
        end
      end
      if self.vx > 0 then
        self.vx = self.vx - (self.currentDrag * dt * self.timeScale)
        if self.vx < 0 then
          self.vx = 0
        end
      end
      self.vx = lume.clamp(self.vx, -self.horizontalMaxSpeed, self.horizontalMaxSpeed)
      local g
      if self.diving then
        g = self.diveGravity
      else
        g = self.gravity
      end
      if self.vy < 0 and self.jumping == false then
        self.vy = self.vy + ((g + self.quickFall) * dt * self.timeScale)
      else
        self.vy = self.vy + (g * dt * self.timeScale)
      end
      local max
      if self.diving then
        max = self.diveMaxSpeed
      else
        max = self.verticalMaxSpeed
      end
      self.vy = lume.clamp(self.vy, -max, max)
      local x, y, cols = self:move(self.vx * dt * self.timeScale, self.vy * dt * self.timeScale)
      for _index_0 = 1, #cols do
        local col = cols[_index_0]
        if col.other.__class == Wall or col.other.__class == Player then
          if col.normal.x ~= 0 then
            self.vx = 0
          end
          if col.normal.y ~= 0 then
            self.vy = 0
            if col.normal.y < 0 then
              self.onGround = true
              self.canDoubleJump = true
            end
          end
        end
      end
      local w, h
      x, y, w, h = self.world:getRect(self)
      if y > HEIGHT + h then
        self.world:update(self, x, -h)
      elseif y < -h then
        self.world:update(self, x, HEIGHT + h)
      end
      if (not self.won) and self.time >= 60 then
        self.won = true
        game.signal.emit('player-win', self.playerNum)
      end
      if math.abs(self.vx) < 10 then
        self.animation.walk:gotoFrame(1)
        self.animation.run:gotoFrame(1)
      end
      if math.abs(self.vx) > 10 then
        self.facingDirection = lume.sign(self.vx)
      end
      if self.vy > 0 then
        self.animation.jump:gotoFrame(3)
      end
      self.animation.walk:update(dt * self.timeScale * (math.abs(self.vx) / self.horizontalMaxSpeed) ^ .3)
      self.animation.run:update(dt * self.timeScale * (math.abs(self.vx) / self.horizontalMaxSpeed) ^ .3)
      self.animation.jump:update(dt * self.timeScale)
      do
        local _with_0 = self.particle.dive
        local _
        _, _, w, h = self.world:getRect(self)
        x, y = self:getCenter()
        _with_0:setPosition(x, y + h / 2)
        if self.onGround or not self.diving then
          _with_0:stop()
        else
          _with_0:start()
        end
        _with_0:update(dt)
      end
      do
        local _with_0 = self.particle.bubble
        _with_0:setPosition(self:getCenter())
        _with_0:update(dt)
        return _with_0
      end
    end,
    clearSignals = function(self)
      game.signal.remove('player-walk', self.walkRegistry)
      game.signal.remove('player-jump', self.jumpRegistry)
      return game.signal.remove('player-end-jump', self.endJumpRegistry)
    end,
    draw = function(self)
      local x, y = self:getCenter()
      do
        local _with_0 = love.graphics
        _with_0.setColor(255, 255, 255, 255)
        _with_0.draw(self.particle.bubble)
        _with_0.setColor(255, 255, 255, 255)
        if self.onGround then
          if math.abs(self.vx) / self.horizontalMaxSpeed > 0.6 then
            self.animation.run:draw(image.characters, x, y - 7, 0, 1 * self.facingDirection, 1, 16, 16)
          else
            self.animation.walk:draw(image.characters, x, y - 7, 0, 1 * self.facingDirection, 1, 16, 16)
          end
        else
          self.animation.jump:draw(image.characters, x, y - 7, 0, 1 * self.facingDirection, 1, 16, 16)
        end
        _with_0.setColor(255, 255, 255, 255)
        _with_0.draw(self.particle.dive)
        local text = tostring(self.time)
        if text:find('%.') then
          text = text:match('(.*%.%d)')
        end
        _with_0.setColor(0, 0, 0, 255)
        _with_0.printCentered(text, font.small, x + 1, y + 17)
        _with_0.setColor(255, 255, 255, 255)
        _with_0.printCentered(text, font.small, x, y + 16)
        self.playerLabel:draw()
        return _with_0
      end
    end,
    drawDebug = function(self)
      if false then
        local x, y, w, h = self.world:getRect(self)
        do
          local _with_0 = love.graphics
          _with_0.setColor(255, 255, 255, 100)
          _with_0.rectangle('fill', x, y, w, h)
          return _with_0
        end
      end
    end
  }
  _base_0.__index = _base_0
  setmetatable(_base_0, _parent_0.__base)
  local _class_0 = setmetatable({
    __init = function(self, world, x, y, playerNum)
      self.playerNum = playerNum
      _parent_0.__init(self, world, x, y, 16, 16)
      self.filter = function(self, other)
        if other.__class == Wall or other.__class == Player then
          return 'slide'
        end
      end
      self.vx = 0
      self.vy = 0
      self.currentDrag = 0
      self.onGround = false
      self.onGroundPrevious = false
      self.canDoubleJump = true
      self.jumping = false
      self.diving = false
      self.time = 0
      self.won = false
      self.hasBubble = false
      self.timeScale = 1
      self.gravity = 1500
      self.verticalMaxSpeed = 600
      self.diveGravity = 5000
      self.diveMaxSpeed = 800
      self.quickFall = 3000
      self.walkAcceleration = 800
      self.horizontalDrag = 2000
      self.horizontalMaxSpeed = 300
      self.baseJumpPower = 450
      self.doubleJumpPower = 350
      self.additionalJumpPower = 100
      self.bubbleSpeedMultiplier = 1
      local character
      if self.playerNum == 1 then
        character = 'guy3'
      end
      if self.playerNum == 2 then
        character = 'guy2'
      end
      self.animation = {
        walk = animation[character].walk:clone(),
        run = animation[character].run:clone(),
        jump = animation[character].jump:clone()
      }
      self.facingDirection = 1
      self.drawDepth = 100
      self.particle = {
        dive = love.graphics.newParticleSystem(image.particle, 100),
        bubble = love.graphics.newParticleSystem(image.particle, 100)
      }
      do
        local _with_0 = self.particle.dive
        _with_0:setEmissionRate(200)
        _with_0:setParticleLifetime(0.2)
        _with_0:setSpeed(100)
        _with_0:setLinearAcceleration(0, -2500)
        _with_0:setSpread(math.pi * 2)
        _with_0:setSizes(2, 0)
        _with_0:setColors(255, 255, 255, 255, 0, 0, 255, 255, 0, 0, 255, 0)
        _with_0:stop()
      end
      do
        local _with_0 = self.particle.bubble
        _with_0:setEmissionRate(10)
        _with_0:setParticleLifetime(1)
        _with_0:setSpeed(50)
        _with_0:setSpread(math.pi * 2)
        _with_0:setColors(100, 100, 255, 255, 100, 100, 255, 0)
        _with_0:stop()
      end
      x, y = self:getCenter()
      self.playerLabel = UIText('P' .. self.playerNum, font.small, x, y - 25)
      do
        local _with_0 = self.playerLabel
        _with_0.shadowDistance = 1
        _with_0.sx, _with_0.sy = 4, 4
        if self.playerNum == 1 then
          _with_0.color = {
            255,
            100,
            100,
            255
          }
        end
        if self.playerNum == 2 then
          _with_0.color = {
            100,
            100,
            255,
            255
          }
        end
      end
      self.timer.add(.15, function()
        return self.tween:to(self.playerLabel, .25, {
          sx = 1,
          sy = 1
        })
      end)
      self.timer.add(1.25, function()
        self.tween:to(self.playerLabel, .25, {
          y = y - 50
        }):ease('quartin')
        self.tween:to(self.playerLabel.color, .25, {
          [4] = 0
        }):ease('quadout')
        return self.tween:to(self.playerLabel.shadowColor, .25, {
          [4] = 0
        }):ease('quadout')
      end)
      self.walkSoundTimer = 1
      self.walkRegistry = game.signal.register('player-walk', function(playerNum, dt, v)
        if playerNum == self.playerNum then
          return self:walk(dt, v)
        end
      end)
      self.jumpRegistry = game.signal.register('player-jump', function(playerNum)
        if playerNum == self.playerNum then
          return self:jump()
        end
      end)
      self.endJumpRegistry = game.signal.register('player-end-jump', function(playerNum)
        if playerNum == self.playerNum then
          return self:endJump()
        end
      end)
      self.diveRegistry = game.signal.register('player-dive', function(playerNum)
        if playerNum == self.playerNum then
          return self:dive()
        end
      end)
      self.endDiveRegistry = game.signal.register('player-end-dive', function(playerNum)
        if playerNum == self.playerNum then
          return self:endDive()
        end
      end)
    end,
    __base = _base_0,
    __name = "Player",
    __parent = _parent_0
  }, {
    __index = function(cls, name)
      local val = rawget(_base_0, name)
      if val == nil then
        return _parent_0[name]
      else
        return val
      end
    end,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  if _parent_0.__inherited then
    _parent_0.__inherited(_parent_0, _class_0)
  end
  Player = _class_0
end
