game = {
  enter = function(self, previous, mapName)
    self.mapName = mapName
    self.updateInterval = 1 / 120
    self.updateTimer = self.updateInterval
    self.signal = signal.new()
    self.timer = timer.new()
    self.inputManager = InputManager()
    self.hud = Hud()
    self.rounds = 0
    self.p1score = 0
    self.p2score = 0
    self.canvas = love.graphics.newCanvas(WIDTH, HEIGHT)
    self.signal.register('player-win', function(playerNum)
      if playerNum == 1 then
        self.p1score = self.p1score + 1
      elseif playerNum == 2 then
        self.p2score = self.p2score + 1
      end
      if self.p1score == 2 or self.p2score == 2 then
        self.signal.emit('match-end', playerNum)
        sound.voiceMatchOver:play()
        return self.timer.add(2, function()
          return gamestate.switch(mapSelect)
        end)
      else
        self.signal.emit('round-end', playerNum)
        sound.voiceRoundOver:play()
        return self.timer.add(2, function()
          return self:nextRound()
        end)
      end
    end)
    return self:nextRound()
  end,
  nextRound = function(self, mapName)
    self.rounds = self.rounds + 1
    if self.map then
      self.map:clearSignals()
    end
    self.map = Map(self.mapName)
    self.signal.emit('game-countoff')
    sound.voiceReady:play()
    return self.timer.add(1.2, function()
      self.signal.emit('game-start')
      return sound.voiceGo:play()
    end)
  end,
  update = function(self, dt)
    self.updateTimer = self.updateTimer - dt
    while self.updateTimer <= 0 do
      self.updateTimer = self.updateTimer + self.updateInterval
      self.timer.update(self.updateInterval)
      self.inputManager:update(self.updateInterval)
      self.map:update(self.updateInterval)
      self.hud:update(self.updateInterval)
    end
  end,
  keypressed = function(self, key)
    return self.inputManager:keypressed(key)
  end,
  keyreleased = function(self, key)
    return self.inputManager:keyreleased(key)
  end,
  draw = function(self)
    self.canvas:clear(0, 0, 0, 255)
    self.canvas:renderTo(function()
      self.map:draw()
      return self.hud:draw()
    end)
    do
      local _with_0 = love.graphics
      _with_0.setColor(255, 255, 255, 255)
      _with_0.draw(self.canvas, 0, 0, 0, _with_0.getHeight() / HEIGHT, _with_0.getHeight() / HEIGHT)
      return _with_0
    end
  end
}
